try:
    from confluent_kafka import Producer, Consumer, KafkaError
    from confluent_kafka import avro
    from confluent_kafka.avro import AvroProducer, AvroConsumer
    from confluent_kafka.avro.serializer import SerializerError
except ImportError:
    raise ImportError("confluent_kafka isn't installed. Run: pip install -U confluent_kafka[avro]")

from divinegift import logger


class ProducerNotSetError(Exception):
    pass


class ConsumerNotSetError(Exception):
    pass


class KafkaClient:
    def __init__(self):
        self.producer = None
        self.avro_producer = False
        self.consumer = None
        self.avro_consumer = False

    def set_producer(self, **configs):
        self.producer = Producer(**configs)

    def set_producer_avro(self, value_schema: str, **configs):
        self.producer = AvroProducer({**configs}, default_value_schema=avro.loads(value_schema))
        self.avro_producer = True

    def close_producer(self):
        if self.producer:
            self.producer.close()
            self.producer = None
            self.avro_producer = False
        else:
            raise ProducerNotSetError('Set consumer before!')

    def set_consumer(self, **configs):
        self.consumer = Consumer(**configs)

    def set_consumer_avro(self, **configs):
        self.consumer = AvroConsumer({**configs})
        self.avro_consumer = True

    def close_consumer(self):
        if self.consumer:
            self.consumer.close()
            self.consumer = None
            self.avro_consumer = False
        else:
            raise ConsumerNotSetError('Set consumer before!')

    def delivery_report(self, err, msg):
        """ Called once for each message produced to indicate delivery result.
            Triggered by poll() or flush(). """
        if err is not None:
            logger.log_info(f'Message delivery failed: {err}')
        else:
            logger.log_info(f'Message delivered to {msg.topic()} [{msg.partition()}]')

    def send_message(self, topic, messages):
        if self.producer is not None:
            if isinstance(messages, list):
                for message in messages:
                    self.producer.poll(0)
                    if isinstance(message, str):
                        self.producer.produce(topic=topic, value=message.encode('utf-8'), callback=self.delivery_report)
                    else:
                        self.producer.produce(topic=topic, value=message, callback=self.delivery_report)
            else:
                self.producer.poll(0)
                if isinstance(messages, str):
                    self.producer.produce(topic=topic, value=messages.encode('utf-8'), callback=self.delivery_report)
                else:
                    self.producer.produce(topic=topic, value=messages, callback=self.delivery_report)
            self.producer.flush()
        else:
            raise ProducerNotSetError('Set producer before!')

    def read_messages(self, topic):
        if self.consumer:
            self.consumer.subscribe([topic])

            while True:
                if self.avro_consumer:
                    try:
                        msg = self.consumer.poll(10)
                    except SerializerError as e:
                        logger.log_err(f"Message deserialization failed for {msg}: {e}")
                        continue
                    if msg is None:
                        continue
                    if msg.error():
                        logger.log_err(f"AvroConsumer error: {msg.error()}")
                        continue
                    logger.log_info(f'Message recieved from {msg.topic()} [{msg.partition()}]')
                    logger.log_debug(f'Received message: {msg.value()}')
                    yield msg.value()
                else:
                    msg = self.consumer.poll(10)
                    if msg is None:
                        continue
                    if msg.error():
                        logger.log_err(f"Consumer error: {msg.error()}")
                        continue
                    logger.log_info(f'Message recieved from {msg.topic()} [{msg.partition()}]')
                    logger.log_debug(f'Received message: {msg.value().decode("utf-8")}')
                    yield msg.value().decode('utf-8')
        else:
            raise ConsumerNotSetError('Set consumer before!')


if __name__ == '__main__':
    pass



from string import Template
import os
from deprecation import deprecated
try:
    from sqlalchemy import create_engine, MetaData, Table
except ImportError:
    raise ImportError("sqlalchemy isn't installed. Run: pip install -U sqlalchemy")


class MethodNotAllowedError(Exception):
    pass


class Connection:
    def __init__(self, db_conn: dict, do_initialize=True):
        self.db_conn = db_conn

        self.engine = None
        self.conn = None
        self.metadata = None

        if do_initialize:
            self.set_conn()

    def set_conn(self):
        """
        Create connection for SQLAlchemy
        :param db_conn: DB connection (user, password, host, port, db_name, dialect)
        :return: Engine, Connection, Metadata
        """
        if self.db_conn.get('dialect') == 'mssql+pytds':
            from sqlalchemy.dialects import registry
            registry.register("mssql.pytds", "sqlalchemy_pytds.dialect", "MSDialect_pytds")
        if self.db_conn.get('db_host') and self.db_conn.get('db_port'):
            connect_str = '{dialect}://{db_user}:{db_pass}@{db_host}:{db_port}/{db_name}'.format(**self.db_conn)
        else:
            connect_str = '{dialect}://{db_user}:{db_pass}@{db_name}'.format(**self.db_conn)
        self.engine = create_engine(connect_str)
        self.conn = self.engine.connect()
        self.metadata = MetaData()

    def set_raw_conn(self):
        """
        Create raw connection for SQLAlchemy
        :param db_conn: DB connection (user, password, host, port, db_name, dialect)
        :return: Connection
        """
        if self.db_conn.get('dialect') == 'mssql+pytds':
            from sqlalchemy.dialects import registry
            registry.register("mssql.pytds", "sqlalchemy_pytds.dialect", "MSDialect_pytds")
        if self.db_conn.get('db_host') and self.db_conn.get('db_port'):
            connect_str = '{dialect}://{db_user}:{db_pass}@{db_host}:{db_port}/{db_name}'.format(**self.db_conn)
        else:
            connect_str = '{dialect}://{db_user}:{db_pass}@{db_name}'.format(**self.db_conn)    
        self.engine = create_engine(connect_str)
        self.conn = self.engine.raw_connection()
        self.metadata = MetaData()

    def get_conn(self, fields = 'conn'):
        if isinstance(fields, str):
            return self.__dict__.get(fields)
        elif isinstance(fields, list):
            return tuple([self.__dict__.get(x) for x in fields])
        else:
            raise MethodNotAllowedError('This type of fields is not allowed')

    def close_conn(self):
        self.conn.close()

    @staticmethod
    def get_sql(filename: str, encoding: str = 'utf-8'):
        """
        Get sql string from file
        :param filename: File name
        :param encoding: Encoding of file
        :return: String with sql
        """
        with open(filename, 'r', encoding=encoding) as file:
            sql = file.read()
        return sql

    def get_data(self, sql: str, encoding: str = 'utf-8', print_script=False, **kwargs):
        """
        Get raw data from sql data as dict
        :param sql: File with sql which need to execute
        :param db_conn: DB connect creditions
        :param encoding: Encoding of file
        :param kwargs: List with additional data
        :return: Dictionary
        """
        if os.path.exists(sql):
            script_t = Template(self.get_sql(sql, encoding))
        else:
            script_t = Template(sql)
        script = script_t.safe_substitute(**kwargs)
        
        if print_script:
            print(script)

        res = self.conn.execute(script)
        ress = [dict(row.items()) for row in res]

        return ress

    def get_data_row(self, sql: str, index: int = 0, encoding: str = 'utf-8', **kwargs):
        """
        Get raw data from sql data as dict
        :param sql: File with sql which need to execute
        :param db_conn: DB connect creditions
        :param index: index of returning row
        :param encoding: Encoding of file
        :param kwargs: List with additional data
        :return: Dictionary
        """
        if os.path.exists(sql):
            script_t = Template(self.get_sql(sql, encoding))
        else:
            script_t = Template(sql)
        script = script_t.safe_substitute(**kwargs)

        res = self.conn.execute(script)
        ress = [dict(row.items()) for row in res]

        try:
            ress = ress[index]
        except:
            ress = None

        return ress

    def run_script(self, sql: str, encoding: str = 'utf-8', **kwargs):
        """
        Run custom script
        :param sql: File with sql which need to execute
        :param db_conn: DB connect creditions
        :param encoding: Encoding of file
        :param kwargs: List with additional data
        :return: None
        """    
        if os.path.exists(sql):
            script_t = Template(self.get_sql(sql, encoding))
        else:
            script_t = Template(sql)
        script = script_t.safe_substitute(**kwargs)

        self.conn.execute(script)


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def get_conn(db_conn: dict):
    conn_obj = Connection(db_conn)
    conn_obj.set_conn()
    return conn_obj.get_conn(['engine', 'conn', 'metadata'])


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def get_raw_conn(db_conn: dict):
    conn_obj = Connection(db_conn)
    conn_obj.set_raw_conn()
    return conn_obj.get_conn('conn')


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def get_sql(filename: str, encoding: str = 'utf-8'):
    """
    Get sql string from file
    :param filename: File name
    :param encoding: Encoding of file
    :return: String with sql
    """
    file = open(filename, 'r', encoding=encoding)
    sql = file.read()
    file.close()
    return sql


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def get_data(sql: str, db_conn, encoding: str = 'utf-8', print_script=False, **kwargs):
    conn_obj = Connection(db_conn)
    if isinstance(db_conn, dict):
        conn_obj.set_conn()
    else:
        conn_obj.conn = db_conn
    ress = conn_obj.get_data(sql, encoding, print_script, **kwargs)

    return ress


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def get_data_row(sql: str, db_conn: dict, index: int = 0, encoding: str = 'utf-8', **kwargs):
    conn_obj = Connection(db_conn)
    if isinstance(db_conn, dict):
        conn_obj.set_conn()
    else:
        conn_obj.conn = db_conn
    ress = conn_obj.get_data_row(sql, index, encoding, **kwargs)

    return ress


@deprecated(deprecated_in='1.3.9', current_version='1.3.9', details='Use class Connection instead')
def run_script(sql: str, db_conn: dict, encoding: str = 'utf-8', **kwargs):
    conn_obj = Connection(db_conn)
    if isinstance(db_conn, dict):
        conn_obj.set_conn()
    else:
        conn_obj.conn = db_conn
    conn_obj.run_script(sql, encoding, **kwargs)


if __name__ == '__main__':
    pass



from divinegift import logger
from divinegift import main
from divinegift import cipher
from mailer import Mailer, Message
import requests
from deprecation import deprecated
from email.header import Header


class Sender:
    def __init__(self):
        self.email_connector = None

    def send_mail(self, msg: str, subject: str, TO: list, CC: list = None, BCC: list = None,
               FROM: str = 'aims.noreply@s7.ru', HOST: str = 'smtp.s7.ru', usr='aims.noreply@s7.ru', pwd=None,
               charset: str = 'utf-8', IS_HTML: bool = True, attachments: object = None, mimetype: str = None):
        """
        Sending email
        :param msg: Message body
        :param subject: Message subject
        :param TO: List of recipients
        :param CC: List of copy recipients
        :param BCC: List of shadow copy recipients
        :param FROM: Sender email
        :param HOST: smtp server
        :param usr: user
        :param pwd: password
        :param charset: Encoding charset
        :param IS_HTML: Will it be HTML or plain text
        :param attachments: Attachments
        :param mimetype: Mimetipe of attachments
        :return:
        """
        message = Message(From=FROM,
                          To=TO,
                          Cc=CC,
                          Bcc=BCC,
                          charset=charset)
        message.Subject = Header(subject.encode('utf-8'), 'UTF-8').encode()
        if IS_HTML:
            message.Html = msg
        else:
            message.Body = msg
        if attachments:
            if type(attachments) == list:
                for file in attachments:
                    try:
                        message.attach(file, mimetype=mimetype, charset=charset)
                    except Exception as ex:
                        logger.log_err(f'Could not attach file: {file}')
            elif type(attachments) == str:
                try:
                    message.attach(attachments)
                except Exception as ex:
                    logger.log_err(f'Could not attach file: {attachments}')
            else:
                logger.log_warning('There is incorrect type of variable attachments')
        if not pwd:
            cipher_ = cipher.get_cipher(b'-2HWUYgoHnOC74Jpc_nUEyXPSYd0R-ZNF4Ur6eTh8Nk=')
            passwd = cipher.decrypt_str(
                b'gAAAAABdCfQXcKZtMjLOwoZLOhSF1Dtul1gwi0IGuiYXJadV03DzQoR8ybrZm4MLt9tHvzYKRRvKW2f9j-lBjRSWMwbpAsj7Pw==',
                cipher_)
        else:
            passwd = pwd

        if self.email_connector is None:
            self.email_connector = Mailer(HOST, usr=usr, pwd=passwd)
        try:
            self.email_connector.send(message)
        except Exception as ex:
            logger.log_err('Error while sending email')
            self.email_connector = None

    def send_telegram(self, message: str, chat_id: int = 161680036, subject: str = None):
        """
            Send a telegram message
            :param message: Message
            :param chat_id: Id of chat where msg will be sent
            :param subject: Subject of message
            :return: None
            """
        URL = 'https://api.telegram.org/bot'  # URL на который отправляется запрос
        TOKEN = '456941934:AAGZMmXJE4VyLagIkVY7qMG0doASxU7f8ac'  # токен вашего бота, полученный от @BotFather
        data = {'chat_id': chat_id,
                'text': (('Тема сообщения: ' + subject + '\n') if subject else '') + 'Сообщение: ' + message}

        try:
            requests.post(URL + TOKEN + '/sendMessage', data=data)  # запрос на отправку сообщения
        except:
            logger.log_err('Send message error')

    def send_slack(self, message: str, webhook: str = None, channel: str = 'aims_integrations',
                   username: str = 'aims_notifier',
                   icon_url: str = None):
        """
        Send message to slack
        :param message: Message
        :param webhook: WebHook URL to sending
        :return: None
        """
        if not webhook:
            webhook = 'https://mattermost.s7.aero/hooks/71ra7afrgjytfq4j5wm4o6x6jo'
        data = {
            'text': message,
            'username': username,
            'channel': channel,
        }
        if icon_url:
            data.update({'icon_url': icon_url})
        try:
            requests.post(webhook, json=data, headers={'content-type': 'application/json'})
        except:
            logger.log_err('Send message error')


def send_email(msg: str, subject: str, TO: list, CC: list = None, BCC: list = None, 
               FROM: str = 'aims.noreply@s7.ru', HOST: str = 'smtp.s7.ru', usr='aims.noreply@s7.ru', pwd=None,
               charset: str = 'utf-8', IS_HTML: bool = True, attachments: object = None, mimetype: str = None):
    sender = Sender()
    sender.send_mail(msg, subject, TO, CC, BCC, FROM, HOST, usr, pwd, charset, IS_HTML, attachments, mimetype)



def send_telegram(message: str, chat_id: int = 161680036, subject: str = None):
    sender = Sender()
    sender.send_telegram(message, chat_id, subject)


def send_slack(message: str, webhook: str = None, channel: str = 'aims_integrations', username: str = 'aims_notifier',
               icon_url: str = None):
    sender = Sender()
    sender.send_slack(message, webhook, channel, username, icon_url)


"""
def auth_vk(login, password):
    # Авторизоваться как человек
    vk = vk_api.VkApi(login=login, password=password)
    vk.auth()
    # Авторизоваться как сообщество
    #vk = vk_api.VkApi(token='a94dd2ef02952a0606fd37f2d1fb11b2d456c034c7671c2b3fab8c3f660474062b9e253c78597d9248469')

    return vk


def send_vk(vk, message, chat_id='8636128', mode='private'):
    #vk = auth_vk()
    if mode == 'private':
        vk.method('messages.send', {'user_id': chat_id, 'message': message})
    elif mode == 'chat':
        vk.method('messages.send', {'peer_id': chat_id, 'message': message})
    elif mode == 'group':
        vk.method('messages.send', {'user_ids': chat_id, 'message': message})
"""

if __name__ == '__main__':
    pass
